/***************************************************************************
 *
 * GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008,2009,2010 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *  version: QAT_CRYPTO_DRIVER.L.0.1.0-14
 *
 ***************************************************************************/
 /*
 * Shim for AES_GCM
 * 
 */
 
#include <linux/errno.h>

#include <linux/scatterlist.h>
#include <crypto/scatterwalk.h>
#include <linux/random.h>

#include <crypto/aead.h>

 
#include "icp_aes_gcm.h"
#include "icp_netkey_shim.h"
#include "cpa_cy_sym.h"

static void symCallback(void *pCallbackTag,
                 CpaStatus status,
                 const CpaCySymOp operationType,
                 void *pOpData,
                 CpaBufferList *pDstBuffer,
                 CpaBoolean verifyResult)
{
   	icp_aead_op_data_t *pCbData = (icp_aead_op_data_t *)pCallbackTag;
	struct aead_request *pReq = NULL; 
	int res = -EBADMSG;
	
	dprintk("%s: callbackTag = %p, status = %u\n", __FUNCTION__, pCallbackTag, status);

    if (NULL == pCbData)
	{
        dprintk("callbackTag is null, status = %d\n", status);
        return;
    }

	pReq = pCbData->pReq;
	
    if (NULL == pOpData)
	{
        dprintk("pOpData is null, status = %d\n", status);
		pReq->base.complete(&(pReq->base), res);
		kfree(pCbData->pAuthTag);
		free_opData_to_cache(pCbData);
        return;
    }

    if (NULL == pDstBuffer)
	{
        dprintk("pDstBuffer is null, status = %d\n", status);
		pReq->base.complete(&(pReq->base), res);
		kfree(pCbData->pAuthTag);
		free_opData_to_cache(pCbData);
		return;
    }

    if (CPA_STATUS_SUCCESS != status)
	{
        dprintk("symCallback failed, status = %d\n", status);
		pReq->base.complete(&(pReq->base), res);
		kfree(pCbData->pAuthTag);
		free_opData_to_cache(pCbData);
    }


	if (verifyResult == CPA_TRUE)
	{
		dprintk("%s: verifyResult == CPA_TRUE (%d)\n", __FUNCTION__, verifyResult);
		res = 0;
	}
	else 
	{
		dprintk("%s: verifyResult == CPA_FALSE (%d)\n", __FUNCTION__, verifyResult);
	}


	#ifdef __DEBUG
	{
		int i = 0;
		dprintk("pDstBuffer: \n");
		for (i = 0; i < pDstBuffer->numBuffers; i++) 
		{
			icp_hexdump(pDstBuffer->pBuffers[i].pData, pDstBuffer->pBuffers[i].dataLenInBytes);
		}
	}
	#endif
	
	dprintk("%s pReq %p\n", __FUNCTION__, pReq);
	dprintk("%s pReq->src %p\n", __FUNCTION__, pReq->src);
	dprintk("%s pReq->dst %p\n", __FUNCTION__, pReq->dst);
	dprintk("%s pCbData->pAuthTag %p\n", __FUNCTION__, pCbData->pAuthTag); 
	
	if (pCbData->cipherDirection == CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT)
	{
	/* copy the tag at the end of the src buffer*/
		scatterwalk_map_and_copy(pCbData->pAuthTag, pReq->src, pReq->cryptlen,
									AES_GCM_AUTH_TAG_LEN, 1);	
	}

    dprintk("%s: pReq->base.complete(%p, 0)\n", __FUNCTION__, pReq);
    pReq->base.complete(&(pReq->base), res);

	kfree(pCbData->pAuthTag);
	free_opData_to_cache(pCbData);
	dprintk("%s: done. \n", __FUNCTION__);

}



static CpaStatus aes_gcm_initSession(icp_aead_tfm_desc_t *pTfm_desc,
									 const CpaCySymCipherDirection cipherDirection)
									 
									 
{
	CpaCySymSessionSetupData sessionSetupData = {0};
	Cpa32U sessionCtxSize = 0;
	CpaStatus status = CPA_STATUS_SUCCESS;
	CpaCySymSessionCtx pSessionCtx = NULL;

    sessionSetupData.sessionPriority = CPA_CY_PRIORITY_HIGH;
    sessionSetupData.symOperation = CPA_CY_SYM_OP_ALGORITHM_CHAINING;
    sessionSetupData.cipherSetupData.cipherAlgorithm = CPA_CY_SYM_CIPHER_AES_GCM;
	sessionSetupData.cipherSetupData.pCipherKey = pTfm_desc->pKey;
	sessionSetupData.cipherSetupData.cipherKeyLenInBytes = pTfm_desc->keylen;
	
    sessionSetupData.hashSetupData.hashAlgorithm = CPA_CY_SYM_HASH_AES_GCM;
	sessionSetupData.hashSetupData.hashMode = CPA_CY_SYM_HASH_MODE_AUTH;
	sessionSetupData.hashSetupData.authModeSetupData.authKey = pTfm_desc->pKey;
	sessionSetupData.hashSetupData.authModeSetupData.authKeyLenInBytes = pTfm_desc->keylen;
	
	sessionSetupData.hashSetupData.authModeSetupData.aadLenInBytes = pTfm_desc->assoclen;
	sessionSetupData.hashSetupData.digestResultLenInBytes = pTfm_desc->auth_size;
	
	
	sessionSetupData.cipherSetupData.cipherDirection = cipherDirection;
	

	dprintk("digestResultLenInBytes %d\n", sessionSetupData.hashSetupData.digestResultLenInBytes);
	dprintk("AADLen %d\n", pTfm_desc->assoclen);
	
	/* Determine size of session context to allocate */
    status = cpaCySymSessionCtxGetSize(CPA_INSTANCE_HANDLE_SINGLE,
                                       &sessionSetupData,
                                       &sessionCtxSize);
	
	if (status != CPA_STATUS_SUCCESS)
	{
		dprintk("cpaCySymSessionCtxGetSize failed\n");
		return status;
	}
	
	pSessionCtx = kmalloc(sessionCtxSize, GFP_ATOMIC);
    if (NULL == pSessionCtx)
	{
        dprintk("sessionCtx - Out of Memory\n");
        return CPA_STATUS_FAIL;
    }
	
	/* Initialize the cipher enc session */
    status = cpaCySymInitSession(CPA_INSTANCE_HANDLE_SINGLE,
                                 symCallback,
								 &sessionSetupData,
								 pSessionCtx);
								
    if (CPA_STATUS_SUCCESS != status)
	{
        dprintk("enc: cpaCySymInitSession failed, status = %d\n", status);
		kfree(pSessionCtx);
        return status;
    }
	
	if (cipherDirection == CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT)
	{
		pTfm_desc->pEncrypt_session_ctx = pSessionCtx;
	}
	else
	{
		pTfm_desc->pDecrypt_session_ctx = pSessionCtx;
	}
	
	return status;
}
									 


 static int perform_aes_gcm(struct aead_request *pReq, Cpa8U *pIv, const CpaCySymCipherDirection cipherDirection)
{
	CpaStatus status = CPA_STATUS_SUCCESS;
    struct crypto_aead *pAead = crypto_aead_reqtfm(pReq);
    struct crypto_tfm *pTfm = crypto_aead_tfm(pAead);
    icp_aead_tfm_desc_t *pTfm_desc = NULL;
	icp_aead_op_data_t *pOp_data = NULL;
	CpaCySymSessionCtx pSessionCtx = NULL;
	Cpa32U messageLenToCipherInBytes = 0;
	
	dprintk("%s: pReq = %p, iv = %p\n", __FUNCTION__, pReq, pIv);
	
	pTfm_desc = find_tfm_desc(pAead);
	
    if (NULL == pTfm_desc)
	{
		printk("Unable to allocate tfm descriptor\n");
        return -ENOMEM;
    }
	

	if (cipherDirection == CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT)
	{
		dprintk("%s: direction is ENCRYPT\n", __FUNCTION__);

		if (pTfm_desc->pEncrypt_session_ctx == NULL)
		{
		/* first encryption with this key: need to create the session */
			pTfm_desc->assoclen= pReq->assoclen;
			status = aes_gcm_initSession(pTfm_desc, cipherDirection);
		}
		pSessionCtx = pTfm_desc->pEncrypt_session_ctx;	
	}
	else
	{
		dprintk("%s: direction is DECRYPT\n", __FUNCTION__);
		
		if (pTfm_desc->pDecrypt_session_ctx == NULL)
		{
		/* first decryption with this key: need to create the session */
			pTfm_desc->assoclen= pReq->assoclen;
			status = aes_gcm_initSession(pTfm_desc, cipherDirection);
		}
		pSessionCtx = pTfm_desc->pDecrypt_session_ctx;	
	}
	
	if (status != CPA_STATUS_SUCCESS)
	{
		return -EPERM;
	}
	
	/* the associated data length should always be the same for a given tunnel*/
	if (pTfm_desc->assoclen != pReq->assoclen)
	{
		printk("Error: All packets must have the same associate data length\n");
		return -EPERM;
	}
	
    dprintk("%s: pSessionCtx = %p\n", __FUNCTION__, pSessionCtx);

	pOp_data = alloc_opData_from_cache();
	if (NULL == pOp_data)
	{
		dprintk("%s: failed to allocate pOp_data buffer\n", __FUNCTION__);
		return -ENOMEM;
	}

	
	memcpy(pTfm_desc->pSalt + GCM_SALT_LEN, pIv, AES_GCM_IV_SIZE);
	/* format of the LAC IV (bytes)
	
	  0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 
	|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|--|
	|   salt    |    initial value      | padding  1|
	-------------------------------------------------
	
	*/
	

	pOp_data->pAuthTag = kmalloc(AES_GCM_AUTH_TAG_LEN, GFP_ATOMIC);
	if (pOp_data->pAuthTag == NULL)
	{
		free_opData_to_cache(pOp_data);
		printk("Unable to allocate memory for authentication tag\n");
		return -ENOMEM;
	}
	
	pOp_data->cipherDirection = cipherDirection;
	pOp_data->pReq = pReq;

	dprintk("%s: pReq->assoclen = %u\n", __FUNCTION__, pReq->assoclen);
	dprintk("%s: pReq->cryptlen = %u\n", __FUNCTION__, pReq->cryptlen);

    /* Setup CpaCySymOpData */
    pOp_data->symOpData.packetType = CPA_CY_SYM_PACKET_TYPE_FULL;
    pOp_data->symOpData.pSessionCtx = pSessionCtx;

    pOp_data->symOpData.pIv = pTfm_desc->pSalt;
    pOp_data->symOpData.ivLenInBytes = LAC_IV_LEN; 

	messageLenToCipherInBytes = pReq->cryptlen;
	if (cipherDirection == CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT)
	{
		/* ctyptlen contains the length of the ciphertext and the length of the authsize;
		we want to decrypt only the ciphertext*/
		messageLenToCipherInBytes = pReq->cryptlen - pTfm_desc->auth_size;
		
		/* copy the auth tag in pAuthTag buffer */
		scatterwalk_map_and_copy(pOp_data->pAuthTag, pReq->src, messageLenToCipherInBytes,
									AES_GCM_AUTH_TAG_LEN, 0);		
	}
	
	pOp_data->symOpData.cryptoStartSrcOffsetInBytes = 0;
    pOp_data->symOpData.messageLenToCipherInBytes = messageLenToCipherInBytes;

	pOp_data->symOpData.hashStartSrcOffsetInBytes = 0;
	pOp_data->symOpData.messageLenToHashInBytes = pReq->assoclen;
	pOp_data->symOpData.pDigestResult = pOp_data->pAuthTag;

    pOp_data->symOpData.pAdditionalAuthData = sg_virt(pReq->assoc);

    pOp_data->symOpData.digestVerify = CPA_FALSE;
	/* set to TRUE to ensure that teh callback function will be successful even if we don't request
	to verify the result*/
	pOp_data->verifyResult = CPA_TRUE;
	
	if (cipherDirection == CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT)
	{
        	pOp_data->symOpData.digestVerify = CPA_TRUE;
	}

	icp_init_bufflist_from_src(pOp_data, pReq->src);

    if (pReq->dst == pReq->src)
	{
        dprintk("%s: inplace -> dst=%p, src=%p\n", __FUNCTION__, pReq->dst, pReq->src);
		
		dprintk("VerifyResult %d\n", pOp_data->verifyResult);

        dprintk("%s: cpaCySymPerformOp()\n", __FUNCTION__);

        status = cpaCySymPerformOp(CPA_INSTANCE_HANDLE_SINGLE,
                                   (void *)pOp_data,
                                    &(pOp_data->symOpData),
                                    &(pOp_data->srcBufferList),
                                    &(pOp_data->srcBufferList),
                                    &(pOp_data->verifyResult));
        if (CPA_STATUS_SUCCESS != status) 
		{
            dprintk("cpaCySymPerformOp failed. (status = %d)\n", status);
		    free_opData_to_cache(pOp_data);
			kfree(pOp_data->pAuthTag);
        	return -EPERM;
        }
        dprintk("%s: cpaCySymPerformOp done.\n", __FUNCTION__);
    }
	else
	{
		dprintk("%s: Out-Of-Place operation not supported\n", __FUNCTION__);
		free_opData_to_cache(pOp_data);
		kfree(pOp_data->pAuthTag);
		return -EINVAL;
	}


    if (pTfm->__crt_alg->cra_flags & CRYPTO_ALG_ASYNC)
	{
		/* the memory will be freed in the callback function*/
        dprintk("%s: set up for async - returning -EINPROGRESS\n", __FUNCTION__);
		dprintk("%s: done\n", __FUNCTION__);
        return -EINPROGRESS;
    }
	else 
	{
		dprintk("%s: synchronoud mode not supported\n", __FUNCTION__);
		free_opData_to_cache(pOp_data);
		kfree(pOp_data->pAuthTag);
		return -EINVAL;
	}

}


int encrypt_aes_gcm(struct aead_request *pReq)
{
	dprintk("%s: encrypt_aes_gcm(pReq=%p)\n", __FUNCTION__, pReq);
	return perform_aes_gcm(pReq, pReq->iv, CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT);
}

int geniv_encrypt_aes_gcm(struct aead_givcrypt_request *pReq)
{
		
	dprintk("%s: geniv_encrypt_aes_gcm(pReq=%p)\n", __FUNCTION__, pReq);

	get_random_bytes(pReq->giv, AES_GCM_IV_SIZE);
	return perform_aes_gcm(&pReq->areq, pReq->giv, CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT);
}

int decrypt_aes_gcm(struct aead_request *pReq)
{
	dprintk("%s: decrypt_aes_gcm(pReq=%p)\n", __FUNCTION__, pReq);
	return perform_aes_gcm(pReq, pReq->iv, CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT);
}

int geniv_decrypt_aes_gcm(struct aead_givcrypt_request *pReq)
{
	dprintk("%s: geniv_decrypt_aes_gcm(req=%p)\n", __FUNCTION__, pReq);	
	return perform_aes_gcm(&pReq->areq, pReq->giv, CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT);
}

int setkey_aes_gcm(struct crypto_aead *pAead, const Cpa8U *pKey, unsigned int key_len)
{
    icp_aead_tfm_desc_t *pTfm_desc = NULL;
	
	dprintk("%s:, key=%p, key_len=%u\n", __FUNCTION__, pKey, key_len);
	
	/* Add new tfm desc */
    pTfm_desc = add_tfm_desc(pAead);
    if (NULL == pTfm_desc)
	{	
		printk("%s: Unable to allocate tfm descriptor\n",  __FUNCTION__);
        return -ENOMEM;
	}
	
	/* the key that we receive contains 16 bytes for the key itself and 4 bytes for 
	the salt that will be used to generate the IV */
	
	pTfm_desc->keylen = key_len - GCM_SALT_LEN;
	/* allocate enough memory to store the key and the LAC IV */
	pTfm_desc->pKey = kzalloc(pTfm_desc->keylen + LAC_IV_LEN , GFP_ATOMIC);
	
	if (pTfm_desc->pKey == NULL)
	{
		 dprintk("%s: Unable to allocate the key\n", __FUNCTION__);
		 return -ENOMEM;
	}
	

	/*  The salt starts at the end of the key*/
	pTfm_desc->pSalt = pTfm_desc->pKey + pTfm_desc->keylen;
	/* set last bit to 1 (padding info required by the spec) */
	pTfm_desc->pSalt[LAC_IV_LEN-1] = 1;
	
	/* copy the full buffer: the key and the salt*/
	memcpy(pTfm_desc->pKey, pKey, key_len);
	
	#ifdef __DEBUG
	dprintk("Key and salt: ");
	icp_hexdump(pTfm_desc->pKey, key_len);
	#endif
	
	pTfm_desc->pEncrypt_session_ctx = NULL;
	pTfm_desc->pDecrypt_session_ctx = NULL;
	
	return 0;
}

