/***************************************************************************
 *
 * GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008,2009,2010 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *  version: QAT_CRYPTO_DRIVER.L.0.1.0-14
 *
 ***************************************************************************/

/*
 * icp_netkey.c
 *
 * This is an implementation of Linux Kernel Crypto API shim that uses
 * the Intel's Quick Assist API. The focus here is IPsec, (Netkey stack).
 * 
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/crypto.h>
#include <linux/scatterlist.h>
#include <linux/rwsem.h>

#include <crypto/aead.h>
#include <crypto/algapi.h> 

#include "icp_sym_chaining.h"
#include "icp_aes_gcm.h"
#include "icp_netkey_shim.h" 
#include "cpa.h"
#include "cpa_cy_sym.h"
#include "cpa_cy_im.h"

static icp_aead_tfm_desc_t *g_pTfm_desc_list = NULL;
static icp_aead_tfm_desc_t *g_pTfm_desc_list_head = NULL;

static struct kmem_cache *pOpDataCache = NULL;
static struct kmem_cache *pMetaCache = NULL;

static struct rw_semaphore tfm_desc_semaphore;

/* add the auth size to the tfm descriptor*/
static int qat_setauthsize(struct crypto_aead *pAead, unsigned int auth_size)
{
  icp_aead_tfm_desc_t *pTfm_desc = NULL;

   dprintk("%s: crypto_aead, auth_size=%u\n", __FUNCTION__,auth_size);
   
    pTfm_desc = add_tfm_desc(pAead);
    if (NULL == pTfm_desc)
	{
		printk("%s: Unable to allocate tfm descriptor\n", __FUNCTION__);
        return -ENOMEM;
    }

   pTfm_desc->auth_size = auth_size;

  return 0;
}

/* free the pointer if null*/
static void icp_kfree(void *pPtr)
{
    if (pPtr != NULL)
	{
        printk("%s: ptr = %p\n", __FUNCTION__, pPtr);
        kfree(pPtr);
    }
}

/* delete the tfm descriptor associated with this aead */
static void del_tfm_desc(const struct crypto_aead* const pAead)
{
    icp_aead_tfm_desc_t *pCurr = NULL;
    CpaStatus status = CPA_STATUS_SUCCESS;
    for (pCurr = g_pTfm_desc_list_head; pCurr != NULL; pCurr = pCurr->pNext)
	{
		if (pCurr->pAead == pAead)
		{
			down_write(&tfm_desc_semaphore);
            ICP_REMOVE_ELEMENT_FROM_LIST(pCurr, g_pTfm_desc_list, g_pTfm_desc_list_head);
			up_write(&tfm_desc_semaphore);
			
			if (pCurr->pEncrypt_session_ctx != NULL)
			{
				status = cpaCySymRemoveSession(CPA_INSTANCE_HANDLE_SINGLE, pCurr->pEncrypt_session_ctx);
			
				if (CPA_STATUS_SUCCESS != status)
				{
					printk("cpaCySymRemoveSession failed. (status = %d)\n", status);
					/* the session could not be removed, its memory can't be freed */
				}
				else
				{
					icp_kfree(pCurr->pEncrypt_session_ctx);
					dprintk("cpaCySymRemoveSession for Encryption was successful. (status = %d)\n", status);
				}
			}
			
			if (pCurr->pDecrypt_session_ctx != NULL)
			{
				status = cpaCySymRemoveSession(CPA_INSTANCE_HANDLE_SINGLE, pCurr->pDecrypt_session_ctx);
			
				if (CPA_STATUS_SUCCESS != status)
				{
				/* the session could not be removed, its memory can't be freed */					
					printk("cpaCySymRemoveSession failed. (status = %d)\n", status);
				}
				else
				{
					icp_kfree(pCurr->pDecrypt_session_ctx);
					dprintk("cpaCySymRemoveSession for Decryption was successful. (status = %d)\n", status);					
				}
			}

            icp_kfree(pCurr);
            dprintk("%s: tfm_desc deleted\n", __FUNCTION__);
            break;
        }
     }
}


static void  aead_auth_exit(struct crypto_tfm *pTfm)
{
	del_tfm_desc((struct crypto_aead *)(pTfm));
}


static CpaStatus convertScatterListToBufferList (CpaBufferList*  pBufferList,
											     const struct scatterlist* const pSrc)
{
struct scatterlist* pCurr = (struct scatterlist*) pSrc;
	while ((pCurr != NULL) && (pBufferList->numBuffers < ICP_MAX_NUM_BUFFERS))
    {
	/* pBufferList->numBuffers contains the number of buffers that have already been initialised*/
        pBufferList->pBuffers[pBufferList->numBuffers].pData = sg_virt(pCurr);
        pBufferList->pBuffers[pBufferList->numBuffers].dataLenInBytes = pCurr->length;
		
		dprintk("%s: sg_virt: pCurr->length = %u\n", __FUNCTION__, pCurr->length);
		dprintk("%s: pBufferList->pBuffers[%d].pData = %p\n",
				__FUNCTION__,
				pBufferList->numBuffers,
				pBufferList->pBuffers[pBufferList->numBuffers].pData);
#ifdef __DEBUG	
		icp_hexdump(pBufferList->pBuffers[pBufferList->numBuffers].pData, 
			pBufferList->pBuffers[pBufferList->numBuffers].dataLenInBytes);
#endif

	    pBufferList->numBuffers++;
        pCurr = sg_next(pCurr);
    }
	
	if (pBufferList->numBuffers == ICP_MAX_NUM_BUFFERS)
	{
		printk("Packet too large\n");
		return CPA_STATUS_FAIL;
	}
	return CPA_STATUS_SUCCESS;
	
}

#ifdef __DEBUG
void icp_hexdump(char *ptr, int len)
{
    int ctr;

    for (ctr = 0; ctr < len; ctr++)
	{
        printk("0x%02X ", ptr[ctr] & 0xff);
        if (!((ctr+1) % 16))
		{
            printk("\n");
		}
    }
    printk("\n");
}
#endif

inline icp_aead_tfm_desc_t *find_tfm_desc(const struct crypto_aead* const pAead)
{
    icp_aead_tfm_desc_t *pCurr = NULL;
	down_read(&tfm_desc_semaphore);
		
    for (pCurr = g_pTfm_desc_list_head; pCurr != NULL; pCurr = pCurr->pNext)
	{
        if (pCurr->pAead == pAead)
		{
            dprintk("%s: found tfm_desc = %p\n", __FUNCTION__, pCurr);
			up_read(&tfm_desc_semaphore);
            return pCurr;
         }
    }
	up_read(&tfm_desc_semaphore);
    return NULL;
}


icp_aead_tfm_desc_t *add_tfm_desc(struct crypto_aead* const pAead)
{
    icp_aead_tfm_desc_t *pCurr = NULL;

    pCurr = find_tfm_desc(pAead);
    if (NULL != pCurr)
	{
        dprintk("%s: tfm_desc already exist for this aead\n", __FUNCTION__);
        return pCurr;
    }

    pCurr = kzalloc(sizeof(icp_aead_tfm_desc_t), GFP_ATOMIC);
    if (NULL == pCurr)
	{
        printk("%s: unable to allocate memory\n", __FUNCTION__);
        return NULL;
    }

    pCurr->pAead = pAead;
	
	down_write(&tfm_desc_semaphore);
    ICP_ADD_ELEMENT_TO_END_OF_LIST(pCurr, g_pTfm_desc_list, g_pTfm_desc_list_head);
	up_write(&tfm_desc_semaphore);

    dprintk("%s: new tfm_desc added %p\n", __FUNCTION__, pCurr);
    return pCurr;
}

inline void * alloc_opData_from_cache(void)
{
	return kmem_cache_alloc(pOpDataCache, GFP_ATOMIC);
}

inline void free_opData_to_cache(icp_aead_op_data_t *pOp_data)
{
	kmem_cache_free(pOpDataCache, pOp_data);
    kmem_cache_free(pMetaCache, pOp_data->srcBufferList.pPrivateMetaData);
}

inline void * allocMetaData(void)
{
	return kmem_cache_zalloc(pMetaCache, GFP_ATOMIC);
}

CpaStatus icp_init_bufflist_from_assoc_iv_src(icp_aead_op_data_t* const pOp_data,
                                              const struct scatterlist* const pAssoc,
                                              const struct scatterlist* const pSrc,
								              const Cpa8U* const pIv,
								              const Cpa32U ivLen)
{
    CpaBufferList *pBufflist = &(pOp_data->srcBufferList);
	
	pBufflist->pPrivateMetaData = allocMetaData();
	if (NULL == pBufflist->pPrivateMetaData) {
		printk("%s: failed to allocate meta buffer\n", __FUNCTION__);
		return CPA_STATUS_FAIL;
	}

	/* 2 buffers will be intialised before calling convertScatterListToBufferList:
	one for pAssoc and one for pIV*/
    pBufflist->numBuffers = 2;

    dprintk("%s: bufflist->numBuffers = %u\n", __FUNCTION__, pBufflist->numBuffers);


    pBufflist->pBuffers = pOp_data->srcFlatBuffer;

    pBufflist->pBuffers[0].pData = sg_virt((struct scatterlist*)pAssoc);
    pBufflist->pBuffers[0].dataLenInBytes = pAssoc->length;

    dprintk("%s: sg_virt: pAssoc->length = %u\n", __FUNCTION__, pAssoc->length);


    dprintk("%s: pBufflist->pBuffers[0].pData = %p\n", __FUNCTION__, pBufflist->pBuffers[0].pData);
#ifdef __DEBUG
    icp_hexdump(pBufflist->pBuffers[0].pData, pBufflist->pBuffers[0].dataLenInBytes);
#endif

    pBufflist->pBuffers[1].pData = (Cpa8U*) pIv;
    pBufflist->pBuffers[1].dataLenInBytes = ivLen;

    dprintk("%s: sg_virt: ivLen = %u\n", __FUNCTION__, ivLen);

    dprintk("%s: pBufflist->pBuffers[1].pData = %p\n", __FUNCTION__, pBufflist->pBuffers[1].pData);
#ifdef __DEBUG
    icp_hexdump(pBufflist->pBuffers[1].pData, pBufflist->pBuffers[1].dataLenInBytes);
#endif
	
	return convertScatterListToBufferList(pBufflist, pSrc);
}

CpaStatus icp_init_bufflist_from_src(icp_aead_op_data_t* const pOp_data,
                                      const struct scatterlist* const pSrc)
{
    CpaBufferList *pBufflist = &(pOp_data->srcBufferList);

	pBufflist->pPrivateMetaData = allocMetaData();
	if (NULL == pBufflist->pPrivateMetaData) {
		printk("%s: failed to allocate meta buffer\n", __FUNCTION__);
		return CPA_STATUS_FAIL;
	}
	
	
    pBufflist->pBuffers = pOp_data->srcFlatBuffer;
	/* no buffers have been initialised before calling convertScatterListToBufferList */
    pBufflist->numBuffers = 0;

	return convertScatterListToBufferList(pBufflist, pSrc);
}

static struct crypto_alg aead_authenc_3des_hmac_sha1 = {
        .cra_name               =       "authenc(hmac(sha1),cbc(des3_ede))",
        .cra_driver_name        =       "icp_aead",
        .cra_priority           =       ICP_AES_ASYNC_PRIORITY,
        .cra_flags              =       CRYPTO_ALG_TYPE_AEAD|CRYPTO_ALG_GENIV|CRYPTO_ALG_ASYNC,
        .cra_blocksize          =       DES3_BLOCK_SIZE /*8*/,
        .cra_ctxsize            =       0,
        .cra_type               =       &crypto_aead_type,
        .cra_module             =       THIS_MODULE,
        .cra_list               =       LIST_HEAD_INIT(aead_authenc_3des_hmac_sha1.cra_list),
		.cra_exit               =       aead_auth_exit,
        .cra_u                  =       {
                .aead = {
			.ivsize       = DES3_BLOCK_SIZE /*8*/,
        	.maxauthsize  = SHA1_DIGEST_SIZE /*20*/,
			.setkey       = setkey_3des_hmac_sha1,
			.setauthsize  = qat_setauthsize,
			.encrypt      = encrypt_3des_hmac_sha1,
			.decrypt      = decrypt_3des_hmac_sha1,
			.givencrypt   = geniv_encrypt_3des_hmac_sha1,
			.givdecrypt   = geniv_decrypt_3des_hmac_sha1,
                }
        }
};


static struct crypto_alg aead_authenc_aes_cbc_hmac_sha1 = {
        .cra_name               =       "authenc(hmac(sha1),cbc(aes))",
        .cra_driver_name        =       "icp_aead",
        .cra_priority           =       ICP_AES_ASYNC_PRIORITY,
        .cra_flags              =       CRYPTO_ALG_TYPE_AEAD|CRYPTO_ALG_GENIV|CRYPTO_ALG_ASYNC,
        .cra_blocksize          =       AES_BLOCK_SIZE /*16*/,
        .cra_ctxsize            =       0,
        .cra_type               =       &crypto_aead_type,
        .cra_module             =       THIS_MODULE,
        .cra_list               =       LIST_HEAD_INIT(aead_authenc_aes_cbc_hmac_sha1.cra_list),
		.cra_exit               =       aead_auth_exit,
        .cra_u                  =       {
                .aead = {
			.ivsize       = AES_BLOCK_SIZE /*16*/,
        	.maxauthsize  = SHA1_DIGEST_SIZE /*20*/,
			.setkey       = setkey_aes_cbc_hmac_sha1,
			.setauthsize  = qat_setauthsize,
			.encrypt      = encrypt_aes_cbc_hmac_sha1,
			.decrypt      = decrypt_aes_cbc_hmac_sha1,
			.givencrypt   = geniv_encrypt_aes_cbc_hmac_sha1,
			.givdecrypt   = geniv_decrypt_aes_cbc_hmac_sha1,
                }
        }
};


static struct crypto_alg aead_authenc_aes_gcm = {
        .cra_name               =       "rfc4106(gcm(aes))",
        .cra_driver_name        =       "icp_aead",
        .cra_priority           =       ICP_AES_ASYNC_PRIORITY,
        .cra_flags              =       CRYPTO_ALG_TYPE_AEAD|CRYPTO_ALG_GENIV|CRYPTO_ALG_ASYNC,
        .cra_blocksize          =       AES_BLOCK_SIZE /*16*/,
        .cra_ctxsize            =       0,
        .cra_type               =       &crypto_aead_type,
        .cra_module             =       THIS_MODULE,
        .cra_list               =       LIST_HEAD_INIT(aead_authenc_aes_gcm.cra_list),
		.cra_exit               =       aead_auth_exit,
        .cra_u                  =       {
                .aead = {
			.ivsize       = AES_GCM_IV_SIZE, /*8*/
        	.maxauthsize  = AES_GCM_AUTH_TAG_LEN, /*16*/
			.setkey       = setkey_aes_gcm,
			.setauthsize  = qat_setauthsize,
			.encrypt      = encrypt_aes_gcm,
			.decrypt      = decrypt_aes_gcm,
			.givencrypt   = geniv_encrypt_aes_gcm,
			.givdecrypt   = geniv_decrypt_aes_gcm,
                }
        }
};

static int __init netkey_init(void)
{
    CpaStatus status = CPA_STATUS_SUCCESS;
	Cpa32U bufferListMetaSize = 0;
	int registerStatus =0;

	init_rwsem(&tfm_desc_semaphore);
		
    status = cpaCyStartInstance(CPA_INSTANCE_HANDLE_SINGLE);
    if (CPA_STATUS_SUCCESS != status) {
        printk("cpaCyStartInstance failed. (status = %d)\n", status);
        return -EPERM;
   }

    /* Determine the size of bufferlist pPrivateMetaData */
    status = cpaCyBufferListGetMetaSize(CPA_INSTANCE_HANDLE_SINGLE,
                                            ICP_MAX_NUM_BUFFERS,
                                            &bufferListMetaSize);
    if (CPA_STATUS_SUCCESS != status)
	{
        printk("cpaCyBufferListGetMetaSize failed. (status = %d)\n", status);
		status = cpaCyStopInstance(CPA_INSTANCE_HANDLE_SINGLE);
		if (CPA_STATUS_SUCCESS != status)
		{
             printk("cpaCyStopInstance failed. (status = %d)\n", status);
        }
        return -EPERM;
    }
    dprintk("%s: cpaCyBufferListGetMetaSize = %u\n", __FUNCTION__, bufferListMetaSize);

	/*
	 * Allocation of the OpData includes the allocation space for meta data.
	 * The memory after the opData structure is reserved for this meta data.
	 */

	pOpDataCache = kmem_cache_create("icp_opdata",
					sizeof(icp_aead_op_data_t),
					0, SLAB_HWCACHE_ALIGN,  NULL);
					
	if (NULL == pOpDataCache) {
		printk("%s: unable to allocate OpData cache\n", __FUNCTION__);
		status = cpaCyStopInstance(CPA_INSTANCE_HANDLE_SINGLE);
		if (CPA_STATUS_SUCCESS != status)
		{
             printk("cpaCyStopInstance failed. (status = %d)\n", status);
        }
		return -ENOMEM;
	}

	pMetaCache = kmem_cache_create("icp_meta",
				 bufferListMetaSize,
               			 0, SLAB_HWCACHE_ALIGN, NULL);
	if (NULL == pMetaCache) {
		printk("%s: unable to allocate OpData cache\n", __FUNCTION__);
		kmem_cache_destroy(pOpDataCache);
        pOpDataCache = NULL;
		status = cpaCyStopInstance(CPA_INSTANCE_HANDLE_SINGLE);
		if (CPA_STATUS_SUCCESS != status)
		{
            printk("cpaCyStopInstance failed. (status = %d)\n", status);
		}
		return -ENOMEM;
	}
	
	registerStatus = crypto_register_alg(&aead_authenc_3des_hmac_sha1);
	if (registerStatus != 0)
	{
		printk("Register alg failed for 3DES-HMAC_SHA1\n");
		kmem_cache_destroy(pOpDataCache);
		kmem_cache_destroy(pMetaCache);
		pOpDataCache = NULL;
        pMetaCache = NULL;
		return registerStatus;
	}	
	printk("3DES-HMAC_SHA1 loaded\n");
	
	registerStatus = crypto_register_alg(&aead_authenc_aes_cbc_hmac_sha1);
	if (registerStatus != 0)
	{
		printk("Register alg failed for AES-CBC-HMAC_SHA1\n");
		kmem_cache_destroy(pOpDataCache);
		kmem_cache_destroy(pMetaCache);
		pOpDataCache = NULL;
        pMetaCache = NULL;
		return registerStatus;
	}
	printk("AES-CBC-HMAC_SHA1 loaded\n");
	
	registerStatus = crypto_register_alg(&aead_authenc_aes_gcm);
	if (registerStatus !=0)
	{
		printk("Register alg failed for AES_GCM\n");
		kmem_cache_destroy(pOpDataCache);
		kmem_cache_destroy(pMetaCache);
		pOpDataCache = NULL;
        pMetaCache = NULL;
		return registerStatus;
	}
	printk("AES_GCM loaded\n");
	printk("%s: Intel ICP NetKey Loaded.\n", __FUNCTION__);
	
	return 0;
}

static void __exit netkey_exit(void)
{
    CpaStatus status = CPA_STATUS_SUCCESS;

	printk("%s: crypto_unregister_alg()\n", __FUNCTION__);
	if (crypto_unregister_alg(&aead_authenc_3des_hmac_sha1) != 0)
	{
		printk("Unable to unload 3DES-HMAC_SHA1\n");
	}
	if (crypto_unregister_alg(&aead_authenc_aes_cbc_hmac_sha1) != 0)
	{
		printk("Unable to unload AES-CBC-HMAC_SHA1\n");
	}
	
	if (crypto_unregister_alg(&aead_authenc_aes_gcm) != 0)
	{
		printk("Unable to unload AES_GCM\n");
	}
	
    status = cpaCyStopInstance(CPA_INSTANCE_HANDLE_SINGLE);
    if (CPA_STATUS_SUCCESS != status)
	{
        printk("cpaCyStopInstance failed. (status = %d)\n", status);
    }

     if (NULL != pOpDataCache)
	 {
        kmem_cache_destroy(pOpDataCache);
        pOpDataCache = NULL;
    }
    if (NULL != pMetaCache) {
         kmem_cache_destroy(pMetaCache);
         pMetaCache = NULL;
     }
	
	/* delete all the tfm descriptors */
	while (NULL != g_pTfm_desc_list_head) {
        del_tfm_desc(g_pTfm_desc_list_head->pAead);
    } 
    printk("%s: Intel ICP NetKey Unloaded.\n", __FUNCTION__);
	
}

module_init(netkey_init);
module_exit(netkey_exit);
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("QAT ICP NETKEY");
MODULE_AUTHOR("Intel");
