/***************************************************************************
 *
 * GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008,2009,2010 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *  version: QAT_CRYPTO_DRIVER.L.0.1.0-14
 *
 ***************************************************************************/

#ifndef ICP_NETKEY_SHIM_H
#define ICP_NETKEY_SHIM_H

#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/crypto.h>

#include "cpa.h"
#include "cpa_cy_sym.h"
#include "cpa_cy_im.h"

#define ICP_MAX_NUM_BUFFERS (7)
/* define the priority in order to be higher than the software implementation*/
#define ICP_AES_ASYNC_PRIORITY  3001

#ifdef __DEBUG
    #define dprintk(...) printk(__VA_ARGS__)
#else
    #define dprintk(...)
#endif

/* Internal structure to store information relevant to a given tunnel*/
typedef struct icp_aead_tfm_desc_s {
        struct crypto_aead *pAead;
        unsigned int       auth_size;
        CpaCySymSessionCtx       pEncrypt_session_ctx;
        CpaCySymSessionCtx       pDecrypt_session_ctx;
		u8*  pKey;
		u8*  pSalt;
		unsigned int keylen;
		unsigned int assoclen;
        struct icp_aead_tfm_desc_s *pPrev;
        struct icp_aead_tfm_desc_s *pNext;
} icp_aead_tfm_desc_t;


/* structure that will be passed to the callback function*/
typedef struct icp_aead_op_data_s {
        struct aead_request *pReq; /* request (needed to set the complete flag) */
		Cpa8U               *pAuthTag; /* auth tag (needed in GCM) */
		CpaCySymCipherDirection cipherDirection;
        CpaCySymOpData      symOpData;
        CpaBufferList       srcBufferList;
        CpaFlatBuffer       srcFlatBuffer[ICP_MAX_NUM_BUFFERS];
        CpaBoolean          verifyResult;
} icp_aead_op_data_t;

/**
* Find a tfm descriptor associated to this aead
* This function is thread safe
* @param[in]  aead
* @ret pointer to the tfm descriptor associated to this aead, NULL if not found 
*/
icp_aead_tfm_desc_t *find_tfm_desc(const struct crypto_aead* const pAead);

/**
* Add a tfm descriptor to the list and associate it to the aead list.
* If there is already a tfm descriptor associated to this aead, this function returns it.
* This funciton is thread safe
* @param[in] aead
* @ret pointer to the new tfm descriptor, NULL if memory allocation fails
*/
icp_aead_tfm_desc_t *add_tfm_desc(struct crypto_aead* const pAead);


/** Returns a pointer to pOpData (allocated from the kernel slab cache) */
 void* alloc_opData_from_cache(void);

/** Free the OpData Memory*/ 
 void free_opData_to_cache(icp_aead_op_data_t *op_data);

/** Allocate Memeory for meta data and returns a pointer to it*/
 void* allocMetaData(void);

/**
* Creates a bufferlist and stores it in the op_data structure.
* This buffer list will contain at least 3 flat buffers: one with the auth data,
* one with IV and at least one with the the source data 
* @param [in/out] pOp_data that will store the buffer list
* @param [in]     pAssoc Associated data scatter list
* @param [in]     pSrc   Source Data scatter list
* @param [in]     pIV    Initial value
* @param [in]     ivLen  Length of IV buffer
* @ret  status (CPA_STATUS SUCCESS or CPA_STATUS_FAIL)  
*/
CpaStatus icp_init_bufflist_from_assoc_iv_src(icp_aead_op_data_t* const pOp_data,
											  const struct scatterlist* const pAssoc,
											  const struct scatterlist* const pSrc,
										      const Cpa8U* const pIv,
								              const Cpa32U ivLen);

/**
* Creates a bufferlist and stores it in the op_data structure.
* This buffer list will contain at least one flat buffers with the source data
* @param [in/out] pOp_data that will store the buffer list
* @param [in]     pSrc   Source Data scatter list
* @ret  status (CAP_STATUS SUCCESS or CPA_STATUS_FAIL)  
*/								 
CpaStatus icp_init_bufflist_from_src(icp_aead_op_data_t* const pOp_data,
									const struct scatterlist*  const pSrc);


/**
* Prints the data contained in the buffer in hexadecimal format
* This function is only defined if __DEBUG is enabled
* @param[in] ptr
* @param[in] len length of the buffer
*/								 
#ifdef __DEBUG								 
void icp_hexdump(char *ptr, int len);								 
#endif

/*Macro for adding an element to the tail of a doubly linked list*/
/*The currentptr tracks the tail, and the headptr tracks the head.*/
#define ICP_ADD_ELEMENT_TO_END_OF_LIST(elementtoadd, currentptr, headptr) \
do {                                                                      \
    if (NULL == currentptr)                                               \
    {                                                                     \
        currentptr = elementtoadd;                                        \
        elementtoadd->pNext = NULL;                                       \
        elementtoadd->pPrev = NULL;                                       \
        headptr = currentptr;                                             \
    }                                                                     \
    else                                                                  \
    {                                                                     \
        elementtoadd->pPrev = currentptr;                                 \
        currentptr->pNext = elementtoadd;                                 \
        elementtoadd->pNext = NULL;                                       \
        currentptr = elementtoadd;                                        \
    }                                                                     \
} while(0)

#define ICP_REMOVE_ELEMENT_FROM_LIST(elementtoremove, currentptr, headptr) \
do {                                                                       \
    /*If the previous pointer is not NULL*/                                \
    if (NULL != elementtoremove->pPrev)                                    \
    {                                                                      \
        elementtoremove->pPrev->pNext = elementtoremove->pNext;            \
        if (elementtoremove->pNext)                                        \
        {                                                                  \
            elementtoremove->pNext->pPrev = elementtoremove->pPrev;        \
        }                                                                  \
        else                                                               \
        {                                                                  \
            /* Move the tail pointer backwards */                          \
            currentptr = elementtoremove->pPrev;                           \
        }                                                                  \
    }                                                                      \
    else if (NULL != elementtoremove->pNext)                               \
    {                                                                      \
        /*Remove the head pointer.*/                                       \
        elementtoremove->pNext->pPrev = NULL;                              \
        /*Hence move the head forward.*/                                   \
        headptr = elementtoremove->pNext;                                  \
    }                                                                      \
    else                                                                   \
    {                                                                      \
        /*Remove the final entry in the list. */                           \
        currentptr = NULL;                                                 \
        headptr = NULL;                                                    \
    }                                                                      \
} while(0)


#endif
