/***************************************************************************
 *
 * GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008,2009,2010 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *  version: QAT_CRYPTO_DRIVER.L.0.1.0-14
 *
 ***************************************************************************/

 /*
 * icp_sym_chaining
 * functions needed for AES_CBC_HMAC_SHA1 and 3DES_HMAC_SHA1
 * 
 */
 
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/crypto.h>
#include <crypto/algapi.h>

#include <linux/scatterlist.h>
#include <linux/highmem.h>

#include <crypto/aead.h>

#include <linux/rtnetlink.h>
#include <crypto/authenc.h>
 

#include "icp_sym_chaining.h"
#include "icp_netkey_shim.h"
#include "cpa_cy_sym.h"



static void symCallback(void *pCallbackTag,
                 CpaStatus status,
                 const CpaCySymOp operationType,
                 void *pOpData,
                 CpaBufferList *pDstBuffer,
                 CpaBoolean verifyResult)
{
   	icp_aead_op_data_t *pCbData = (icp_aead_op_data_t *)pCallbackTag;
	struct aead_request *pReq = NULL;
	int res = -EBADMSG;
	
	dprintk("%s: callbackTag = %p, status = %u\n", __FUNCTION__, pCallbackTag, status);

    if (NULL == pCallbackTag)
	{
        dprintk("callbackTag is null, status = %d\n", status);
        return;
    }

	pReq = pCbData->pReq;

    if (NULL == pOpData)
	{
        dprintk("pOpData is null, status = %d\n", status);
		pReq->base.complete(&(pReq->base), res);
		free_opData_to_cache(pCbData);
        return;
    }


	
    if (NULL == pDstBuffer)
	{
        dprintk("pDstBuffer is null, status = %d\n", status);
		pReq->base.complete(&(pReq->base), res);
		free_opData_to_cache(pCbData);
		return;
    }

    if (CPA_STATUS_SUCCESS != status)
	{
        dprintk("symCallback failed, status = %d\n", status);
    }
	else
	{
		if (verifyResult == CPA_TRUE)
		{
			dprintk("%s: verifyResult == CPA_TRUE (%d)\n", __FUNCTION__, verifyResult);
			/* set the result to success*/
			res = 0;
		}
		else 
		{
			dprintk("%s: verifyResult == CPA_FALSE (%d)\n", __FUNCTION__, verifyResult);
		}
	}

	#ifdef __DEBUG
	{
		int i = 0;
		dprintk("pDstBuffer: \n");
			for (i = 0; i < pDstBuffer->numBuffers; i++) 
		{
				icp_hexdump(pDstBuffer->pBuffers[i].pData, pDstBuffer->pBuffers[i].dataLenInBytes);
		}
	}
	#endif


    dprintk("%s: req->base.complete(%p, 0)\n", __FUNCTION__, pReq);
    pReq->base.complete(&(pReq->base), res);

	free_opData_to_cache(pCbData);
	dprintk("%s: done. \n", __FUNCTION__);

}


 
 static int perform_sym_chaining(struct aead_request *pReq,
								Cpa8U *pIv,
								const Cpa32U ivLen,
								const CpaCySymCipherDirection cipherDirection)
{
	CpaStatus status = CPA_STATUS_SUCCESS;
    struct crypto_aead *pAead = crypto_aead_reqtfm(pReq);
    struct crypto_tfm *pTfm = crypto_aead_tfm(crypto_aead_reqtfm(pReq));
    icp_aead_tfm_desc_t *pTfm_desc = NULL;
    icp_aead_op_data_t *pOp_data = NULL;
	CpaCySymSessionCtx pSessionCtx = NULL;
	Cpa32U messageLenToCipherInBytes = 0;
	
	dprintk("%s: req = %p, iv = %p\n", __FUNCTION__, pReq, pIv);
	pTfm_desc = find_tfm_desc(pAead);
	
    if (NULL == pTfm_desc)
	{
		printk("%s: Unable to find tfm descriptor\n", __FUNCTION__);
        return -EPERM;
    }
	

	if (cipherDirection == CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT) {
		dprintk("%s: direction is ENCRYPT\n", __FUNCTION__);
		pSessionCtx = pTfm_desc->pEncrypt_session_ctx;
	}
	else {
		dprintk("%s: direction is DECRYPT\n", __FUNCTION__);
		pSessionCtx = pTfm_desc->pDecrypt_session_ctx;
	}
	
     dprintk("%s: pSessionCtx = %p\n", __FUNCTION__, pSessionCtx);

	pOp_data = alloc_opData_from_cache();
	if (NULL == pOp_data) {
		dprintk("%s: failed to allocate pOp_data buffer\n", __FUNCTION__);
		return -ENOMEM;
	}

	pOp_data->pReq = pReq;

	dprintk("%s: req->assoclen = %u\n", __FUNCTION__, pReq->assoclen);
	dprintk("%s: req->cryptlen = %u\n", __FUNCTION__, pReq->cryptlen);

    /* Setup CpaCySymOpData */
    pOp_data->symOpData.packetType = CPA_CY_SYM_PACKET_TYPE_FULL;
    pOp_data->symOpData.pSessionCtx = pSessionCtx;

    pOp_data->symOpData.pIv = pIv;
    pOp_data->symOpData.ivLenInBytes = ivLen; 

	messageLenToCipherInBytes = pReq->cryptlen;
	
	if (cipherDirection == CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT)
	{
		/* we want to decrypt only the ciphertext - not the authentication tag */
		messageLenToCipherInBytes = pReq->cryptlen - pTfm_desc->auth_size;
	}
	
	/* structure of the message sent to QAT
	
	-------------------------------------------(----)
	| Assoc  |   IV      |      Plaintext     |( T |)
	-------------------------------------------(----)
	                      <messagelentocipher>
	< ------ message len to hash ----------->
	<cryptostartsrcoffset>
	
	The tag T is only present in decrypt mode */
	
    pOp_data->symOpData.cryptoStartSrcOffsetInBytes = pReq->assoclen + ivLen;
    pOp_data->symOpData.messageLenToCipherInBytes = messageLenToCipherInBytes;

	pOp_data->symOpData.hashStartSrcOffsetInBytes = 0;
	pOp_data->symOpData.messageLenToHashInBytes = pReq->assoclen + ivLen + messageLenToCipherInBytes;

	pOp_data->symOpData.pDigestResult = sg_virt(pReq->src) + messageLenToCipherInBytes;

    pOp_data->symOpData.pAdditionalAuthData = NULL;

    pOp_data->symOpData.digestVerify = CPA_FALSE;
	pOp_data->verifyResult = CPA_FALSE;
	
	if (cipherDirection == CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT)
	{
        	pOp_data->symOpData.digestVerify = CPA_TRUE;
	}

	icp_init_bufflist_from_assoc_iv_src(pOp_data, pReq->assoc, pReq->src, pIv, ivLen);

    if (pReq->dst == pReq->src)
	{
        dprintk("%s: inplace -> dst=%p, src=%p\n", __FUNCTION__, pReq->dst, pReq->src);
		
		dprintk("VerifyResult %d\n", pOp_data->verifyResult);

        dprintk("%s: cpaCySymPerformOp()\n", __FUNCTION__);

        status = cpaCySymPerformOp(CPA_INSTANCE_HANDLE_SINGLE,
                                   (void *)pOp_data,
                                    &(pOp_data->symOpData),
                                    &(pOp_data->srcBufferList),
                                    &(pOp_data->srcBufferList),
                                    &(pOp_data->verifyResult));
        if (CPA_STATUS_SUCCESS != status) 
		{
            dprintk("cpaCySymPerformOp failed. (status = %d)\n", status);
			free_opData_to_cache(pOp_data);
        	return -EPERM;
        }
        dprintk("%s: cpaCySymPerformOp done.\n", __FUNCTION__);
    }
	else
	{
		dprintk("%s: Out-Of-Place operation not supported\n", __FUNCTION__);
		free_opData_to_cache(pOp_data);
		return -EINVAL;
	}

    if (pTfm->__crt_alg->cra_flags & CRYPTO_ALG_ASYNC)
	{
        dprintk("%s: set up for async - returning -EINPROGRESS\n", __FUNCTION__);
        return -EINPROGRESS;
    }
	else 
	{
		dprintk("%s: synchronous mode not supported\n", __FUNCTION__);
		free_opData_to_cache(pOp_data);
		return -EINVAL;
	}

}

/* AES-CBC SHA-HMAC*/
int encrypt_aes_cbc_hmac_sha1(struct aead_request *pReq)
{
	dprintk("%s: encrypt_aes_cbc_hmac_sha1(req=%p)\n", __FUNCTION__, pReq);
	return perform_sym_chaining(pReq, pReq->iv, AES_IV_LEN, CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT);
}

int geniv_encrypt_aes_cbc_hmac_sha1(struct aead_givcrypt_request *pReq)
{
	dprintk("%s: geniv_encrypt_aes_cbc_hmac_sha1(req=%p)\n", __FUNCTION__, pReq);
	get_random_bytes(pReq->giv, AES_IV_LEN); 
	return perform_sym_chaining(&pReq->areq, pReq->giv, AES_IV_LEN, CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT);
}

int decrypt_aes_cbc_hmac_sha1(struct aead_request *pReq)
{
	dprintk("%s: decrypt_aes_cbc_hmac_sha1(req=%p)\n", __FUNCTION__, pReq);
	return perform_sym_chaining(pReq, pReq->iv, AES_IV_LEN, CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT);
}

int geniv_decrypt_aes_cbc_hmac_sha1(struct aead_givcrypt_request *pReq)
{
	dprintk("%s: geniv_decrypt_aes_cbc_hmac_sha1(req=%p)\n", __FUNCTION__, pReq);
	return perform_sym_chaining(&pReq->areq, pReq->giv, AES_IV_LEN, CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT);
}


/* 3DES SHA-HMAC*/
int encrypt_3des_hmac_sha1(struct aead_request *pReq)
{
	dprintk("%s: encrypt_3des_hmac_sha1(req=%p)\n", __FUNCTION__, pReq);
	return perform_sym_chaining(pReq, pReq->iv, DES3_IV_LEN, CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT);
}

int geniv_encrypt_3des_hmac_sha1(struct aead_givcrypt_request *pReq)
{
	dprintk("%s: geniv_encrypt_3des_hmac_sha1(req=%p)\n", __FUNCTION__, 
	pReq);
	get_random_bytes(pReq->giv, DES3_IV_LEN); 
	return perform_sym_chaining(&pReq->areq, pReq->giv, DES3_IV_LEN, CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT);
}

int decrypt_3des_hmac_sha1(struct aead_request *pReq)
{
	dprintk("%s: decrypt_3des_hmac_sha1(req=%p)\n", __FUNCTION__, pReq);
	return perform_sym_chaining(pReq, pReq->iv, DES3_IV_LEN, CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT);
}

int geniv_decrypt_3des_hmac_sha1(struct aead_givcrypt_request *pReq)
{
	dprintk("%s: geniv_decrypt_3des_hmac_sha1(req=%p)\n", __FUNCTION__, pReq);
	return perform_sym_chaining(&pReq->areq, pReq->giv, DES3_IV_LEN, CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT);
}



static int setkey_sym_chaining(struct crypto_aead *pAead,
									const Cpa8U *pKey,
									unsigned int keylen,
									CpaCySymCipherAlgorithm cipherAlgorithm,
									CpaCySymHashAlgorithm hashAlgorithm)
{
    icp_aead_tfm_desc_t *pTfm_desc = NULL;
    struct rtattr *pRta = (void *)pKey;
    struct crypto_authenc_key_param *pParam = NULL;
	unsigned int enckeylen;
	unsigned int authkeylen;
	CpaStatus status = CPA_STATUS_SUCCESS;
	
	CpaCySymSessionSetupData sessionSetupData = {0};
	Cpa32U sessionCtxSize = 0;

	/* Add new tfm desc */
    pTfm_desc = add_tfm_desc(pAead);
    if (NULL == pTfm_desc)
	{
		printk("%s: Unable to allocate tfm descriptor\n", __FUNCTION__);
        return -ENOMEM;
	}
	dprintk("%s: key=%p, keylen=%u\n", __FUNCTION__, pKey, keylen);	


	/* The key we receive is the concatenation of:
	- flags (length is RTA_ALIGN(pRta->rta_len))
	- authentication key (length can be computed)
	- cipher key (length is given by the flags)
	*/
     if (!RTA_OK(pRta, keylen))
	{
		crypto_aead_set_flags(pAead, CRYPTO_TFM_RES_BAD_KEY_LEN);
		return -EINVAL;
	}

    if (pRta->rta_type != CRYPTO_AUTHENC_KEYA_PARAM)
	{
		crypto_aead_set_flags(pAead, CRYPTO_TFM_RES_BAD_KEY_LEN);
        return -EINVAL;
    }

     if (RTA_PAYLOAD(pRta) < sizeof(*pParam))
	 {
        crypto_aead_set_flags(pAead, CRYPTO_TFM_RES_BAD_KEY_LEN);
        return -EINVAL;
    }

    pParam = RTA_DATA(pRta);
    enckeylen = be32_to_cpu(pParam->enckeylen);

	dprintk("%s: enckeylen = %u\n", __FUNCTION__, enckeylen);

    pKey += RTA_ALIGN(pRta->rta_len);
    keylen -= RTA_ALIGN(pRta->rta_len);

	dprintk("%s: keylen = %u\n", __FUNCTION__, keylen);

    if (keylen < enckeylen)
	{
        crypto_aead_set_flags(pAead, CRYPTO_TFM_RES_BAD_KEY_LEN);
        return -EINVAL;
    }

	authkeylen = keylen - enckeylen;


    sessionSetupData.sessionPriority = CPA_CY_PRIORITY_HIGH;
    sessionSetupData.symOperation = CPA_CY_SYM_OP_ALGORITHM_CHAINING;
	sessionSetupData.cipherSetupData.cipherAlgorithm = cipherAlgorithm;
	sessionSetupData.cipherSetupData.pCipherKey = (Cpa8U*)pKey + authkeylen;
	sessionSetupData.cipherSetupData.cipherKeyLenInBytes = enckeylen;
	
	sessionSetupData.hashSetupData.hashAlgorithm = hashAlgorithm;
	sessionSetupData.hashSetupData.hashMode = CPA_CY_SYM_HASH_MODE_AUTH;
	sessionSetupData.hashSetupData.authModeSetupData.authKey = (Cpa8U*)pKey;
	sessionSetupData.hashSetupData.authModeSetupData.authKeyLenInBytes = authkeylen;
	sessionSetupData.hashSetupData.digestResultLenInBytes = pTfm_desc->auth_size;
	
	/* encrypt first*/
	sessionSetupData.cipherSetupData.cipherDirection = CPA_CY_SYM_CIPHER_DIRECTION_ENCRYPT;
	sessionSetupData.algChainOrder = CPA_CY_SYM_ALG_CHAIN_ORDER_CIPHER_THEN_HASH;
	
	dprintk("enckeylen %d\n", enckeylen);
	dprintk("authkeylen %d\n", authkeylen);
	dprintk("digestResultLenInBytes %d\n", sessionSetupData.hashSetupData.digestResultLenInBytes); 
	
	/* Determine size of session context to allocate */
    status = cpaCySymSessionCtxGetSize(CPA_INSTANCE_HANDLE_SINGLE,
                                       &sessionSetupData,
                                       &sessionCtxSize);
	
	if (status != CPA_STATUS_SUCCESS)
	{
		dprintk("cpaCySymSessionCtxGetSize failed\n");
		return -EPERM;
	}
	
	pTfm_desc->pEncrypt_session_ctx = kmalloc(sessionCtxSize, GFP_ATOMIC);
    if (NULL == pTfm_desc->pEncrypt_session_ctx )
	{
        dprintk("sessionCtx - Out of Memory\n");
        return -ENOMEM;
    }
	
	/* Initialize the cipher enc session */
    status = cpaCySymInitSession(CPA_INSTANCE_HANDLE_SINGLE,
                                 symCallback,
								 &sessionSetupData,
								 pTfm_desc->pEncrypt_session_ctx);
								
    if (CPA_STATUS_SUCCESS != status)
	{
        dprintk("enc: cpaCySymInitSession failed, status = %d\n", status);
		kfree(pTfm_desc->pEncrypt_session_ctx);
		pTfm_desc->pEncrypt_session_ctx = NULL;
        return -EPERM;
    }
	
	dprintk("Encrypt Session created: %p - size %d\n", pTfm_desc->pEncrypt_session_ctx, sessionCtxSize);
	
	/* now setup the decrypt session */
	sessionSetupData.cipherSetupData.cipherDirection = CPA_CY_SYM_CIPHER_DIRECTION_DECRYPT;
	sessionSetupData.algChainOrder = CPA_CY_SYM_ALG_CHAIN_ORDER_HASH_THEN_CIPHER;
	
	/* Determine size of session context to allocate */
    status = cpaCySymSessionCtxGetSize(CPA_INSTANCE_HANDLE_SINGLE,
                                       &sessionSetupData,
                                       &sessionCtxSize);
	
	if (status != CPA_STATUS_SUCCESS)
	{
		dprintk("cpaCySymSessionCtxGetSize failed\n");
		kfree(pTfm_desc->pEncrypt_session_ctx);
		pTfm_desc->pEncrypt_session_ctx = NULL;
		return -EPERM;
	}
	
	pTfm_desc->pDecrypt_session_ctx = kmalloc(sessionCtxSize, GFP_ATOMIC);
    if (NULL == pTfm_desc->pDecrypt_session_ctx )
	{
        dprintk("sessionCtx - Out of Memory\n");
		kfree(pTfm_desc->pEncrypt_session_ctx);
		pTfm_desc->pEncrypt_session_ctx = NULL;
        return -ENOMEM;
    }
	
	/* Initialize the cipher decrypt session */
    status = cpaCySymInitSession(CPA_INSTANCE_HANDLE_SINGLE,
                                 symCallback,
								 &sessionSetupData,
								 pTfm_desc->pDecrypt_session_ctx);
								
    if (CPA_STATUS_SUCCESS != status)
	{
        dprintk("enc: cpaCySymInitSession failed, status = %d\n", status);
		kfree(pTfm_desc->pEncrypt_session_ctx);
		pTfm_desc->pEncrypt_session_ctx = NULL;
		kfree(pTfm_desc->pDecrypt_session_ctx);
		pTfm_desc->pDecrypt_session_ctx = NULL;
        return -EPERM;
    }
	dprintk("Decrypt Session created: %p - size %d\n", pTfm_desc->pDecrypt_session_ctx, sessionCtxSize);
	return 0;
}

int setkey_aes_cbc_hmac_sha1(struct crypto_aead *pAead,
									const Cpa8U *pKey,
									unsigned int key_len)
									
{
	return setkey_sym_chaining(pAead, pKey, key_len, CPA_CY_SYM_CIPHER_AES_CBC, CPA_CY_SYM_HASH_SHA1);
}

int setkey_3des_hmac_sha1(struct crypto_aead *pAead, const Cpa8U *pKey, unsigned int key_len)
{
	return setkey_sym_chaining(pAead, pKey, key_len, CPA_CY_SYM_CIPHER_3DES_CBC, CPA_CY_SYM_HASH_SHA1);
}
